#!/usr/bin/fades

import datetime
import decimal
import io
import sys

import PyPDF2  # fades

MONTHS = {
    "Enero": 1,
    "Febrero": 2,
    "Marzo": 3,
    "Abril": 4,
    "Mayo": 5,
    "Junio": 6,
    "Julio": 7,
    "Agosto": 8,
    "Septiembre": 9,
    "Setiembre": 9,
    "Setiem.": 9,
    "Octubre": 10,
    "Noviembre": 11,
    "Noviem.": 11,
    "Diciembre": 12,
    "Diciem.": 12,
}


def process(reader):
    filtered = []
    for idx in range(reader.numPages):
        page = reader.getPage(idx)
        lines = page.extractText().split("\n")
        useful = False
        for line in lines:
            if "abonando el pago mínimo" in line or "Usted puede cancelar en cuotas" in line:
                return filtered
            if "LIMITES" in line and "FINANCIACION" in line:
                useful = True
                continue
            if useful:
                filtered.append(line)


def decode_date(srcstr):
    try:
        year, monthname, day = srcstr.split()
    except ValueError:
        return

    year = int("20" + year)
    month = MONTHS[monthname]
    day = int(day)
    return datetime.date(year=year, month=month, day=day)


def encode_date(dt):
    return dt.strftime("%d/%m/%Y") if dt is not None else "-"


def decode_decimal(srcstr):
    if not srcstr:
        srcstr = "0"
    if srcstr[-1] == "-":
        # minus sign is after!
        srcstr = "-" + srcstr[:-1]
    try:
        return decimal.Decimal(srcstr.replace(".", "").replace(",", "."))
    except decimal.InvalidOperation:
        print("ERROR when converting to Decimal:", repr(srcstr))
        exit()


def encode_decimal(n):
    return str(n).replace(".", ",")


def main(filename, total_amount):
    with open(filename, 'rb') as fh:
        data = fh.read()
    data = data.removeprefix(b"[BEGIN]\n")
    pre_filtered = process(PyPDF2.PdfFileReader(io.BytesIO(data)))

    filtered = []
    pre_date = None
    for line in pre_filtered:
        if not line.strip():
            continue
        if "SALDO ANTERIOR" in line or "SU PAGO" in line:
            continue
        if line.startswith("     "):
            # insert the previous date here, if exists
            if pre_date is not None:
                line = pre_date + line[10:]   # 10 is year and month, not full date
        elif not line[:2].isdigit() or line[2] != " ":
            # expecting the date, that starts with two digits like "23 Julio ..."
            continue
        else:
            # get the date for this line and maybe next ones
            pre_date = line[:10]   # 10 is year and month, not full date
        filtered.append(line)

    # separate in columns
    cols_parsing = [
        ("date", 13, decode_date),
        ("id", 7, str),
        ("mark", 3, str),
        ("description", 50, str),
        ("ars", 20, decode_decimal),
        ("usd", 20, decode_decimal),
    ]
    records = []
    for raw in filtered:
        base = 0
        record = {}
        for name, length, func in cols_parsing:
            raw_part = raw[base: base + length].strip()
            record[name] = func(raw_part)
            base += length
        records.append(record)

    # get the total in ARS and in USD, and calculate exchange rate
    ars = sum(record["ars"] for record in records)
    usd = sum(record["usd"] for record in records)
    if usd:
        exchange_rate = (total_amount - ars) / usd
        msg = f"given total={total_amount} ars={ars} usd={usd} xr={exchange_rate}"
        assert round(exchange_rate, 2) == int(exchange_rate), msg
    else:
        exchange_rate = 0

    for record in records:
        record["arsx"] = record["usd"] * exchange_rate

    cols_showing = [
        ("date", encode_date, "fecha"),
        ("id", str, "id"),
        ("mark", str, "marca"),
        ("description", str, "descripción"),
        ("ars", encode_decimal, "ARS"),
        ("arsx", encode_decimal, f"ARS({exchange_rate})"),
        ("usd", encode_decimal, "USD"),
    ]
    print("\t".join(title for _, _, title in cols_showing))
    for record in records:
        parts = [func(record[name]) for name, func, _ in cols_showing]
        print("\t".join(parts))


if len(sys.argv) != 3:
    print("USAGE: proc_tarjeta_santander.py resumen.pdf total_amount")
    exit(1)

filename, total_amount = sys.argv[1:3]
total_amount = abs(decode_decimal(total_amount))

main(filename, total_amount)
